# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Mock service ext."""

from invenio_records_resources.resources import RecordResource
from invenio_records_resources.services import RecordService

from .administration.mock import MockView
from .config import ServiceConfig
from .resource import MockResource


class MockExtension:
    """Mock extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Init app."""
        mock_service = RecordService(ServiceConfig)
        mock_resource = RecordResource(MockResource, mock_service)

        app.extensions["mock-module"] = self
        #  Adds resource's config name as a class attribute.
        self.__dict__[MockView.resource_config] = mock_resource



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Administration exceptions."""

from invenio_i18n import gettext as _


class InvalidResource(Exception):
    """Exception for invalid resources."""

    def __init__(self, resource, view):
        """Initialise error."""
        super().__init__(
            _(
                "Invalid resource {resource} configured for view {view}.".format(
                    resource=resource, view=view
                )
            )
        )


class InvalidExtensionName(KeyError):
    """Exception for invalid extension names."""

    def __init__(self, extension_name):
        """Initialise error."""
        super().__init__(
            _(
                "No extension found with name '{extension_name}' .".format(
                    extension_name=extension_name
                )
            )
        )


class InvalidActionsConfiguration(KeyError):
    """Exception for invalid extension names."""

    def __init__(self):
        """Initialise error."""
        super().__init__(_("Invalid actions configuration, order or schema missing"))


class MissingResourceConfiguration(Exception):
    """Exception for missing resource configurations."""

    def __init__(self, name):
        """Initialise error."""
        super().__init__(
            _(
                "Cannot instantiate resource view {name} without a resource.".format(
                    name=name
                )
            )
        )


class MissingExtensionName(Exception):
    """Exception for missing resource configurations."""

    def __init__(self, name):
        """Initialise error."""
        super().__init__(
            _(
                "Cannot instantiate resource view {name} without an "
                "associated flask extension.".format(name=name)
            )
        )


class MissingDefaultGetView(Exception):
    """Exception for missing default GET views."""

    def __init__(self, name):
        """Initialise error."""
        super().__init__(
            _(
                "Cannot instantiate administration view {name} without a "
                "default GET view".format(name=name)
            )
        )



import pytest
from mock_module.administration.mock import MockView, MockViewAlternate

from invenio_administration.views.base import AdminView


class TestCustomView(AdminView):
    name = "Test Custom View"
    extension_name = "test-custom-view"
    category = "Test category"


def test_view_registration_from_entry_point(
    mock_extension, current_admin_core, test_app
):
    """Test view registration from entry point."""
    # Validate extension is registered
    assert mock_extension is not None

    # Assert view registration
    registered_view = list(
        filter(lambda x: x.view_class == MockView, current_admin_core.views)
    )
    assert len(registered_view) == 1
    assert registered_view[0].view_class == MockView

    # Assert url is registered
    custom_view_rule = [
        x
        for x in test_app.url_map.iter_rules()
        if x.rule == f"{current_admin_core.url}/{MockView.url}"
    ]
    assert len(custom_view_rule) == 1

    # Validate that two views are registered from the same module
    # Assert view registration
    registered_view = list(
        filter(lambda x: x.view_class == MockViewAlternate, current_admin_core.views)
    )
    assert len(registered_view) == 1
    assert registered_view[0].view_class == MockViewAlternate


def test_view_registration_duplicated(current_admin_ext, test_app, mock_extension_name):
    """Test duplicated view registration."""
    dup_cls = MockView
    dup_cls.extension_name = mock_extension_name

    with pytest.raises(ValueError):
        current_admin_ext.register_view(
            MockView, extension_name=dup_cls.extension_name, app=test_app
        )


def test_view_registration_url_naming(test_app, current_admin_core):
    """Test view registration urls with spaces."""
    # Get registered url from app's url Map
    view_filter = [
        v
        for v in current_admin_core.views
        if v.view_class.name == MockViewAlternate.name
    ]
    assert len(view_filter) == 1

    view = view_filter[0]

    custom_view_rule = [
        x
        for x in test_app.url_map.iter_rules()
        if x.endpoint == f"{current_admin_core.endpoint}.{view.view_class.name}"
    ]

    assert len(custom_view_rule) == 1

    registered_url = custom_view_rule[0].rule

    # Test that the registered url is the view's name without whitespaces, lower cased.
    expected_url = "{}/{}".format(
        current_admin_core.url, MockViewAlternate.name.replace(" ", "_").lower()
    )
    assert registered_url == expected_url


def test_view_registration_url_explicit(test_app, current_admin_core):
    """Test url registration for explicit set urls."""
    custom_view_rule = [
        x
        for x in test_app.url_map.iter_rules()
        if x.rule == f"{current_admin_core.url}/{MockView.url}"
    ]
    assert len(custom_view_rule) == 1



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Mock service configuration."""

from invenio_records_resources.services import RecordServiceConfig


class ServiceConfig(RecordServiceConfig):
    """Mock service configuration.

    Needs both configs, with File overwritting the record ones.
    """

    permission_policy_cls = None
    record_cls = None
    schema = None



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Configuration for Invenio-Admin."""

ADMINISTRATION_BASE_TEMPLATE = "invenio_administration/base.html"
"""Admin panel base template.
By default (``None``) uses the Flask-Admin template."""

ADMINISTRATION_APPNAME = "Invenio-Administration"
"""Name of the Flask-Admin app (also the page title of admin panel)."""

ADMINISTRATION_DASHBOARD_VIEW = (
    "invenio_administration.views.dashboard.AdminDashboardView"
)
"""Administration dashboard view class."""

ADMINISTRATION_THEME_BASE_TEMPLATE = "invenio_theme/page.html"
"""Administration base template."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 CERN.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# invenio-administration is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

import pytest
from marshmallow import Schema, fields

from invenio_administration.marshmallow_utils import custom_mapping, jsonify_schema


class CustomField(fields.Field):
    """A custom field class that extends the base Field class."""

    pass


class CustomInheritedField(CustomField):
    """An inherited custom field class derived from CustomField."""

    pass


class UnrecognizedField(fields.Field):
    """A custom field class that is not recognized in the custom mapping."""

    pass


class TestSchema(Schema):
    """Schema class for testing, with various field types."""

    string_field = fields.Str()
    integer_field = fields.Integer()
    custom_field = CustomField()
    inherited_field = CustomInheritedField()


@pytest.fixture
def update_custom_mapping():
    """
    Pytest fixture to temporarily update the custom mapping for field types.
    Restores the original mapping after the test.
    """
    original_mapping = custom_mapping.copy()
    custom_mapping[CustomField] = "custom"
    custom_mapping[CustomInheritedField] = "custom"
    yield
    custom_mapping.clear()
    custom_mapping.update(original_mapping)


def test_jsonify_schema_with_standard_fields(update_custom_mapping):
    """Assert standard field types are correctly identified."""
    schema = TestSchema()
    result = jsonify_schema(schema)
    assert result["string_field"]["type"] == "string"
    assert result["integer_field"]["type"] == "integer"


def test_jsonify_schema_with_custom_field(update_custom_mapping):
    """Assert custom_field is identified as 'custom'."""
    schema = TestSchema()
    result = jsonify_schema(schema)
    assert result["custom_field"]["type"] == "custom"


def test_jsonify_schema_with_inherited_field(update_custom_mapping):
    """
    Assert inherited_field is correctly identified as 'custom'.
    """
    schema = TestSchema()
    result = jsonify_schema(schema)
    assert result["inherited_field"]["type"] == "custom"


def test_jsonify_schema_with_unrecognized_field(update_custom_mapping):
    """
    Assert jsonify_schema raises an exception for an unrecognized field in the schema.
    """

    class UnrecognizedSchema(TestSchema):
        unrecognized_field = UnrecognizedField()

    schema = UnrecognizedSchema()
    with pytest.raises(Exception) as excinfo:
        jsonify_schema(schema)
    assert "Unrecognised schema field" in str(excinfo.value)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Administration menu module."""

import urllib.parse

from flask import request
from invenio_i18n import lazy_gettext as _
from invenio_theme.proxies import current_theme_icons
from speaklater import make_lazy_string

from invenio_administration.permissions import administration_permission


class AdminMenu:
    """Main class for the admin menu."""

    def __init__(self):
        """Constructor."""
        self._menu_items = []

    @property
    def items(self):
        """Return all raw menu items."""
        return self._menu_items

    def register_menu_entries(self, flask_menu_instance, menu_key="admin_navigation"):
        """Register all menu items to a flask menu instance."""
        main_menu = flask_menu_instance.submenu(menu_key)

        # items without category go first and the rest are sorted alphabetically
        ordered_menu_items = sorted(
            self._menu_items,
            key=lambda menu_item: (menu_item.category is not None, menu_item.category),
        )

        for menu_entry in ordered_menu_items:
            category = menu_entry.category
            name = menu_entry.name
            endpoint = menu_entry.endpoint
            order = menu_entry.order
            active_when = menu_entry.active_when
            label = menu_entry.label
            icon = menu_entry.icon
            disabled = menu_entry.disabled

            if disabled():
                continue

            if category:
                category_menu = main_menu.submenu(category)
                category_menu.register(text=category)
                category_menu.submenu(name).register(
                    endpoint=endpoint,
                    text=label,
                    order=order,
                    active_when=active_when or self.sub_content_active_when,
                    icon=icon,
                )
            else:
                main_menu.submenu(name).register(
                    endpoint=endpoint,
                    text=label,
                    order=order,
                    active_when=active_when or self.default_active_when,
                    icon=icon,
                )

    def register_admin_entry(self, current_menu, endpoint):
        """Register administration entry as the last one."""
        current_menu.submenu("profile-admin.administration").register(
            f"{endpoint}.dashboard",
            _(
                "%(icon)s Administration",
                icon=make_lazy_string(
                    lambda: f'<i class="{current_theme_icons.cogs}"></i>'
                ),
            ),
            order=1,
            visible_when=lambda: administration_permission.can(),
        )

    def add_menu_item(self, item, index=None):
        """Add menu item."""
        if not isinstance(item, MenuItem):
            return TypeError("Item should be MenuItem instance.")

        if index:
            self._menu_items[index] = item
            return

        self._menu_items.append(item)

    def add_view_to_menu(self, view, index=None):
        """Add menu item from view."""
        menu_item = MenuItem(
            endpoint=view.endpoint,
            name=view.name,
            category=view.category,
            label=view.menu_label,
            order=view.order,
            icon_key=view.icon,
            disabled=view.disabled,
        )

        self.add_menu_item(menu_item, index)

    @staticmethod
    def default_active_when(self):
        """Default condition for the menu item active state."""
        return request.endpoint == self._endpoint

    @staticmethod
    def sub_content_active_when(self):
        """Condition for menu items with sub content.

        Makes all pages with derivative URL highlight the parent menu.
        """
        menu_url = urllib.parse.urlparse(self.url)
        request_url = urllib.parse.urlparse(request.url_rule.rule)

        return request_url.path == menu_url.path or request_url.path.startswith(
            f"{menu_url.path}/"
        )


class MenuItem:
    """Class for menu item."""

    def __init__(
        self,
        name="",
        endpoint="",
        category="",
        order=0,
        icon_key=None,
        active_when=None,
        label="",
        disabled=lambda x: False,
    ):
        """Constructor."""
        self.name = name
        self.endpoint = endpoint
        self.category = category
        self.order = order
        self.active_when = active_when
        self.icon_key = icon_key
        self.label = label
        self.disabled = disabled

    @property
    def icon(self):
        """Return corresponding template path for icon."""
        if not self.icon_key:
            return None

        return current_theme_icons[self.icon_key]



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Administration views base module."""
from functools import partial

from flask import current_app, render_template, url_for
from flask.views import MethodView
from invenio_search_ui.searchconfig import search_app_config

from invenio_administration.errors import (
    InvalidActionsConfiguration,
    InvalidExtensionName,
    InvalidResource,
    MissingDefaultGetView,
    MissingExtensionName,
    MissingResourceConfiguration,
)
from invenio_administration.marshmallow_utils import jsonify_schema
from invenio_administration.permissions import administration_permission


class AdminView(MethodView):
    """Base view for admin views."""

    extension_name = None
    name = None
    category = None
    template = "invenio_administration/index.html"
    url = None
    menu_label = None
    order = None
    icon = None

    decorators = [administration_permission.require(http_exception=403)]

    def __init__(
        self,
        name=__name__,
        category=None,
        url=None,
        extension_name=None,
        admin=None,
        order=0,
        icon=None,
    ):
        """Constructor."""
        if self.extension_name is None:
            self.extension_name = extension_name

        if self.name is None:
            self.name = name

        if self.category is None:
            self.category = category

        if self.menu_label is None:
            self.menu_label = self.name

        self.administration = admin

        if self.order is None:
            self.order = order

        if self.icon is None:
            self.icon = icon

        self.url = url or self._get_view_url(self.url)

        # Default view
        if self.get is None:
            raise MissingDefaultGetView(self.__class__.__name__)

    @staticmethod
    def disabled():
        """Determine if the view should be disabled."""
        return False

    @property
    def endpoint(self):
        """Get name for endpoint location e.g: 'administration.index'."""
        if self.administration is None:
            return self.name
        return f"{self.administration.endpoint}.{self.name}"

    @classmethod
    def _get_view_extension(cls, extension_name=None):
        """Get the flask extension of the view."""
        try:
            if extension_name:
                return current_app.extensions[extension_name]
            return current_app.extensions[cls.extension_name]
        except KeyError:
            raise InvalidExtensionName(extension_name)

    def _get_view_url(self, url):
        """Generate URL for the view. Override to change default behavior."""
        new_url = url
        if new_url is None:
            if isinstance(self, self.administration.dashboard_view_class):
                new_url = "/"
            else:
                new_url = "/%s" % self.name.lower()
        else:
            if not url.startswith("/"):
                new_url = "/%s" % (url)
        # Sanitize url
        new_url = new_url.replace(" ", "_")
        return new_url

    def _get_template(self):
        return self.template

    def render(self, **kwargs):
        """Render template."""
        kwargs["admin_base_template"] = self.administration.base_template
        return render_template(self._get_template(), **kwargs)

    def get(self):
        """GET view method."""
        return self.render()


class AdminResourceBaseView(AdminView):
    """Base view for admin resources."""

    display_edit = False
    display_delete = False
    resource_config = None
    resource = None
    actions = {}
    schema = None
    api_endpoint = None
    pid_path = "pid"
    title = None
    resource_name = None

    create_view_name = None
    list_view_name = None
    request_headers = {"Accept": "application/json"}

    def __init__(
        self,
        name=__name__,
        category=None,
        url=None,
        extension_name=None,
        admin=None,
        order=0,
        icon=None,
    ):
        """Constructor."""
        super().__init__(name, category, url, extension_name, admin, order, icon)

        if self.extension_name is None:
            raise MissingExtensionName(self.__class__.__name__)
        if self.resource_config is None:
            raise MissingResourceConfiguration(self.__class__.__name__)

    @classmethod
    def set_schema(cls):
        """Set schema."""
        cls.schema = cls.get_service_schema()

    @classmethod
    def set_resource(cls, extension_name=None):
        """Set resource."""
        cls.resource = cls._get_resource(extension_name)

    @classmethod
    def _get_resource(cls, extension_name=None):
        extension_name = cls._get_view_extension(extension_name)
        try:
            return getattr(extension_name, cls.resource_config)
        except AttributeError:
            raise InvalidResource(resource=cls.resource_config, view=cls.__name__)

    @classmethod
    def get_service_schema(cls):
        """Get marshmallow schema of the assigned service."""
        # schema.schema due to the schema wrapper imposed,
        # when the actual class needed
        return cls.resource.service.schema.schema()

    def _schema_to_json(self, schema):
        """Translate marshmallow schema to JSON.

        Provides action payload template for the frontend.
        """
        return jsonify_schema(schema)

    def get_api_endpoint(self):
        """Get search API endpoint."""
        api_url_prefix = current_app.config["SITE_API_URL"]
        slash_tpl = "/" if not self.api_endpoint.startswith("/") else ""

        if not self.api_endpoint.startswith(api_url_prefix):
            return f"{api_url_prefix}{slash_tpl}{self.api_endpoint}"

        return f"{slash_tpl}{self.api_endpoint}"

    def get_actions(self):
        """Get actions attribute."""
        return self.actions

    def serialize_actions(self):
        """Serialize actions for the resource frontend view.

        {"action_name":
            {"text": "Action"
             "payload_schema": schema in json
             "order": 1
             }
         }
        """
        serialized_actions = {}
        for key, value in self.get_actions().items():
            if "payload_schema" and "order" not in value:
                raise InvalidActionsConfiguration

            serialized_actions[key] = {"text": value["text"], "order": value["order"]}
            if value["payload_schema"] is not None:
                serialized_actions[key]["payload_schema"] = self._schema_to_json(
                    value["payload_schema"]()
                )

        return serialized_actions

    def get_list_view_endpoint(self):
        """Returns administration UI list view endpoint."""
        if self.list_view_name:
            return url_for(f"administration.{self.list_view_name}")
        if isinstance(self, AdminResourceListView):
            return url_for(f"administration.{self.name}")

    def get_create_view_endpoint(self):
        """Returns administration UI list view endpoint."""
        if self.create_view_name:
            return url_for(f"administration.{self.create_view_name}")


class AdminResourceDetailView(AdminResourceBaseView):
    """Details view based on given config."""

    display_edit = True
    display_delete = True

    name = None
    item_field_exclude_list = None
    item_field_list = None
    template = "invenio_administration/details.html"
    title = "Resource details"

    def get_context(self, pid_value=None):
        """Create details view context."""
        name = self.name
        schema = self.get_service_schema()
        serialized_schema = self._schema_to_json(schema)
        fields = self.item_field_list
        return {
            "request_headers": self.request_headers,
            "name": name,
            "resource_schema": serialized_schema,
            "fields": fields,
            "exclude_fields": self.item_field_exclude_list,
            "ui_config": self.item_field_list,
            "pid": pid_value,
            "api_endpoint": self.get_api_endpoint(),
            "title": self.title,
            "list_endpoint": self.get_list_view_endpoint(),
            "actions": self.serialize_actions(),
            "pid_path": self.pid_path,
            "display_edit": self.display_edit,
            "display_delete": self.display_delete,
            "list_ui_endpoint": self.get_list_view_endpoint(),
            "resource_name": (
                self.resource_name if self.resource_name else self.pid_path
            ),
        }

    def get(self, pid_value=None):
        """GET view method."""
        return self.render(**self.get_context(pid_value=pid_value))


class AdminFormView(AdminResourceBaseView):
    """Basic form view."""

    form_fields = None
    display_read_only = True

    def get(self, pid_value=None):
        """GET view method."""
        schema = self.get_service_schema()
        serialized_schema = self._schema_to_json(schema)
        form_fields = self.form_fields
        return self.render(
            **{
                "resource_schema": serialized_schema,
                "form_fields": form_fields,
                "pid": pid_value,
                "api_endpoint": self.get_api_endpoint(),
                "title": self.title,
                "list_endpoint": self.get_list_view_endpoint(),
                "ui_config": self.form_fields,
            }
        )


class AdminResourceEditView(AdminFormView):
    """Admin resource edit view."""

    template = "invenio_administration/edit.html"
    title = "Edit resource"


class AdminResourceCreateView(AdminFormView):
    """Admin resource edit view."""

    template = "invenio_administration/create.html"
    title = "Create resource"


class AdminResourceListView(AdminResourceBaseView):
    """List view based on provided resource."""

    template = "invenio_administration/search.html"

    # decides if there is a detail view
    display_read = True
    display_create = True

    # hides searchbar
    display_search = True

    search_config_name = None
    search_facets_config_name = None
    search_sort_config_name = None
    sort_options = {}
    available_facets = {}
    item_field_exclude_list = None
    item_field_list = None
    api_endpoint = None
    item_api_endpoint = None
    title = None

    search_request_headers = {"Accept": "application/json"}

    def get_search_request_headers(self):
        """Get search request headers."""
        return self.search_request_headers

    def get_search_app_name(self):
        """Get search app name."""
        if self.search_config_name is None:
            return f"{self.name.upper()}_SEARCH"
        return self.search_config_name

    def init_search_config(self):
        """Build search view config."""
        return partial(
            search_app_config,
            config_name=self.get_search_app_name(),
            available_facets=current_app.config.get(self.search_facets_config_name),
            sort_options=current_app.config[self.search_sort_config_name],
            endpoint=self.get_api_endpoint(),
            headers=self.get_search_request_headers(),
        )

    def get_sort_options(self):
        """Get search sort options."""
        if not self.sort_options:
            return self.resource.service.config.search.sort_options
        return self.sort_options

    def get_available_facets(self):
        """Get search available facets."""
        if not self.available_facets:
            return self.resource.service.config.search.facets
        return self.available_facets

    def get(self):
        """GET view method."""
        search_conf = self.init_search_config()
        schema = self.get_service_schema()
        serialized_schema = self._schema_to_json(schema)
        return self.render(
            **{
                "search_config": search_conf,
                "title": self.title,
                "name": self.name,
                "resource_schema": serialized_schema,
                "fields": self.item_field_list,
                "display_search": self.display_search,
                "display_create": self.display_create,
                "display_edit": self.display_edit,
                "display_delete": self.display_delete,
                "display_read": self.display_read,
                "actions": self.serialize_actions(),
                "pid_path": self.pid_path,
                "create_ui_endpoint": self.get_create_view_endpoint(),
                "list_ui_endpoint": self.get_list_view_endpoint(),
                "resource_name": (
                    self.resource_name if self.resource_name else self.pid_path
                ),
            }
        )


class AdminResourceViewSet:
    """View set based on resource.

    Provides a list view and a details view given the provided configuration.
    """

    extension_name = None
    name = None
    category = None
    template = "invenio_administration/index.html"
    url = None
    menu_label = None

    resource_config = None
    resource = None

    schema = None
    api_endpoint = None
    pid_path = "pid"
    title = None
    actions = None

    create_view_name = None
    edit_view_name = None
    details_view_name = None
    list_view_name = None

    display_create = False
    display_read = True
    display_edit = False
    display_delete = False

    sort_options = ()
    available_filters = None
    item_field_exclude_list = None
    item_field_list = None

    def list_view(self):
        """List view."""
        pass

    def details_view(self):
        """Details view."""
        pass

    def edit_view(self):
        """Details view."""
        pass

    def create_view(self):
        """Details view."""
        pass



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Administration menu test module."""

from mock_module.administration.mock import MockView


def test_menu_generation(current_admin_menu, current_admin_core, test_app):
    # Retrieve mock module's view from registered views.
    registered_view = list(
        filter(lambda x: x.view_class.name == MockView.name, current_admin_core.views)
    )
    assert len(registered_view) == 1

    mock_view = registered_view[0].view_class
    category = mock_view.category
    name = mock_view.name

    # register menu items on flask-menu instance
    current_admin_core._menu.register_menu_entries(current_admin_menu)
    # default admin navigation menu
    flask_menu_admin_instance = current_admin_menu.submenu("admin_navigation")

    # check if menu category was made correctly and has children
    category_menu = flask_menu_admin_instance.submenu(category)
    assert category_menu.text == category
    assert len(category_menu.children) > 0
    # menu entry from the registered view
    menu_entry_filter = [x for x in category_menu.children if x.text == name]

    # check if correct menu entry made under the category
    assert len(menu_entry_filter) == 1
    menu_entry = menu_entry_filter[0]
    # assert menu_entry._endpoint == test_view.endpoint

    # needs context to generate url from endpoint
    app_context = test_app.app_context()
    test_context = test_app.test_request_context()
    with app_context, test_context:
        assert menu_entry.url == f"{current_admin_core.url}/{mock_view.url}"



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Administration core admin module."""

from flask import Blueprint
from flask_menu import current_menu
from werkzeug.utils import import_string

from invenio_administration.menu import AdminMenu

from .views.base import AdminFormView, AdminResourceDetailView, AdminView


class Administration:
    """Admin views core manager."""

    def __init__(
        self,
        app=None,
        name=None,
        url=None,
        ui_endpoint=None,
        base_template=None,
    ):
        """Constructor.

        :param app: flask application
        :param name: application name, defaults to "Admin"
        :param url: base admin url to register the dashboard view and subviews
        :param dashboard_view: home page view
        :param ui_endpoint: base UI endpoint,
                     leaves flexibility to implement two different admin apps,
                     or to provide custom endpoint
        :param base_template: base admin template.
                     Defaults to "admin/base.html"
        """
        super().__init__()

        self.app = app
        self._views = []
        self._menu = AdminMenu()
        self._menu_key = "admin_navigation"
        self.blueprint = None

        if name is None:
            name = "Administration"
        self.name = name

        self.dashboard_view_class = self.load_admin_dashboard(app)
        self.endpoint = ui_endpoint or "administration"
        self.url = url or "/administration"
        self.base_template = base_template or "invenio_administration/base.html"

        self.create_blueprint()

        if self.dashboard_view_class is not None:
            self._add_dashboard_view()

    def init_menu(self):
        """Finalize app.

        This method will be called within the finalize_app entrypoint.
        """
        self._menu.register_menu_entries(current_menu, self._menu_key)
        self._menu.register_admin_entry(current_menu, self.endpoint)

    def load_admin_dashboard(self, app):
        """Load dashboard view configuration."""
        dashboard_config = app.config["ADMINISTRATION_DASHBOARD_VIEW"]
        dashboard_class = import_string(dashboard_config)
        return dashboard_class

    def create_blueprint(self):
        """Create Flask blueprint."""
        # Create blueprint and register rules
        self.blueprint = Blueprint(
            self.endpoint,
            __name__,
            url_prefix=self.url,
            template_folder="templates",
            static_folder="static",
        )

    @property
    def views(self):
        """Registered admin views."""
        return self._views

    def add_view(self, view, view_instance, *args, **kwargs):
        """Add a view to admin views."""
        # Validate view's class and name's uniqueness.
        if not issubclass(view.view_class, AdminView):
            raise TypeError(f"View class must be of type {AdminView.__name__}")
        if any(v.view_class.name == view.view_class.name for v in self.views):
            raise ValueError(f"View name already registered: {view.view_class.name}")

        self.views.append(view)

        self.blueprint.add_url_rule(
            rule=view_instance.url,
            view_func=view,
        )

        if not isinstance(view_instance, AdminResourceDetailView) and not isinstance(
            view_instance, AdminFormView
        ):
            self._menu.add_view_to_menu(view_instance)

    def _add_dashboard_view(self):
        """Add the admin dashboard view."""
        dashboard_instance = self.dashboard_view_class(
            admin=self, extension_name="invenio-administration"
        )
        dashboard_view = self.dashboard_view_class.as_view(
            self.dashboard_view_class.name,
            admin=self,
            extension_name="invenio-administration",
        )

        self.add_view(dashboard_view, dashboard_instance)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 CERN.
#
# invenio-administration is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module providing extensible administration panel."""

from .ext import InvenioAdministration

__version__ = "2.2.1"

__all__ = ["InvenioAdministration"]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# invenio-administration is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Permissions for administration module."""

from invenio_access import action_factory
from invenio_access.permissions import Permission

administration_access_action = action_factory("administration-access")
administration_permission = Permission(administration_access_action)
